#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <libxml/tree.h>
#include <libxml/parser.h>
#include <glib.h>

#include "main.h"
#include "callbacks.h"
#include "interface.h"
#include "support.h"

gchar *selected_filename;

void
on_main_window_destroy_event           (GtkObject       *object,
                                        gpointer         user_data)
{
	gtk_main_quit ();
}


gboolean
on_main_window_delete_event            (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{

  return FALSE;
}


void
on_new_activate                        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_open_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gnome_app_flash ((GnomeApp *)main_window, "Opening world...");
  
  file_open_dialog = create_file_open_dialog();
  
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION(file_open_dialog)->ok_button),
		      "clicked", GTK_SIGNAL_FUNC (store_filename), NULL);
  
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION(file_open_dialog)->ok_button),
			     "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     (gpointer) file_open_dialog);
  
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION(file_open_dialog)->cancel_button),
			     "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     (gpointer) file_open_dialog);
  
  gtk_widget_show(file_open_dialog);
}

void 
store_filename                        (GtkFileSelection *selector, 
                                       gpointer          user_data) 
{
  selected_filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION(file_open_dialog));
  open_saved_file(selected_filename);
}


void
on_save_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_save_as_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_quit_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_cut_activate                        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_copy_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_paste_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_clear_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_properties_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_preferences_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_about_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	about = create_about ();
	gtk_widget_show(about);
}

void*
get_grandchild_node_value (xmlDocPtr doc, gchar* parentName, gchar* nodeName) {
	xmlNodePtr cur;
	xmlNodePtr temp;
	
	cur = xmlDocGetRootElement(doc);
	
	cur = cur->xmlChildrenNode;
	while (cur != NULL) {
		if ((!xmlStrcmp(cur->name, (const xmlChar *)parentName))) {
			temp = cur;
			break;
		}
		cur = cur->next;
	}
	
	if (temp == NULL) {
		return NULL;
	}
	
	cur = temp->xmlChildrenNode;
	while (cur != NULL) {
		if ((!xmlStrcmp(cur->name, (const xmlChar *)nodeName))) {
			return (char *)xmlNodeListGetString(doc,cur->xmlChildrenNode,1);
		}
		cur = cur->next;
	}
	return NULL;
}
void*
get_child_node_value (xmlDocPtr doc, gchar* nodeName) {
	xmlNodePtr cur;
	
	cur = xmlDocGetRootElement(doc);
	
	cur = cur->xmlChildrenNode;
	while (cur != NULL) {
		if ((!xmlStrcmp(cur->name, (const xmlChar *)nodeName))) {
			return (char *)xmlNodeListGetString(doc,cur->xmlChildrenNode,1);
		}
	}
	return NULL;
}

void
open_saved_file	(gchar* myfile) {
	xmlDocPtr doc;
	xmlNodePtr node, display, character, comment, cur;
	xmlNsPtr ns;
	gchar *DisplayName, *CharacterName, *Password, *AutoConnect;
	char WorldDomain[256];
	char WindowTitle[128];
	struct hostent *h;

	doc = xmlParseFile(myfile);

	if (doc == NULL) {
		xmlFreeDoc(doc);
		return;
	}

	cur = xmlDocGetRootElement(doc);

	if (cur == NULL) {
		xmlFreeDoc(doc);
		return;
	}

	ns = xmlSearchNsByHref(doc, cur, (const xmlChar *)"http://panache.kydance.net/ns/1.0/");
	
	if (ns == NULL) {
		xmlFreeDoc(doc);
		return;
	}
	
	if (xmlStrcmp(cur->name,(const xmlChar *) "world-definition")) {
		printf("Root node name 'world-definition': %s\n",(char *)cur->name);
		xmlFreeDoc(doc);
		return;
	}

	WorldListIndex = WorldListLength;
	WorldListLength++;
	
	WorldList[WorldListIndex].WorldDomain = g_string_new(get_grandchild_node_value(doc,"connection","domain"));
	WorldList[WorldListIndex].DisplayName = g_string_new(get_grandchild_node_value(doc,"world","name"));
	WorldList[WorldListIndex].WorldPort = atoi(get_grandchild_node_value(doc,"connection","port"));
	WorldList[WorldListIndex].WorldType = atoi(get_grandchild_node_value(doc,"world","type"));
	WorldList[WorldListIndex].CharacterName = g_string_new(get_grandchild_node_value(doc,"character","name"));
	WorldList[WorldListIndex].Password = g_string_new(get_grandchild_node_value(doc,"character","password"));
	WorldList[WorldListIndex].AutoConnect = atoi(get_grandchild_node_value(doc,"character","autoconnect"));

	WorldList[WorldListIndex].storedcommand = g_string_new("");

	WorldList[WorldListIndex].io_pane = gtk_vpaned_new ();
	gtk_widget_show (WorldList[WorldListIndex].io_pane);
	gtk_container_add (GTK_CONTAINER (world_view), WorldList[WorldListIndex].io_pane);
	gtk_paned_set_position (GTK_PANED (WorldList[WorldListIndex].io_pane), 450);

  WorldList[WorldListIndex].output_scroller = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (WorldList[WorldListIndex].output_scroller);
  gtk_paned_pack1 (GTK_PANED (WorldList[WorldListIndex].io_pane), WorldList[WorldListIndex].output_scroller, FALSE, TRUE);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (WorldList[WorldListIndex].output_scroller), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);

  WorldList[WorldListIndex].output_text = gtk_text_view_new ();
  create_tags (gtk_text_view_get_buffer (GTK_TEXT_VIEW (WorldList[WorldListIndex].output_text)));

  gtk_text_buffer_get_iter_at_line(gtk_text_view_get_buffer(GTK_TEXT_VIEW(WorldList[WorldListIndex].output_text)), &WorldList[WorldListIndex].last_line_iter,gtk_text_buffer_get_line_count(gtk_text_view_get_buffer(GTK_TEXT_VIEW(WorldList[WorldListIndex].output_text))));
  gtk_text_buffer_create_mark(gtk_text_view_get_buffer (GTK_TEXT_VIEW (WorldList[WorldListIndex].output_text)),"last_line",&WorldList[WorldListIndex].last_line_iter,FALSE);
  
  gtk_widget_modify_base(WorldList[WorldListIndex].output_text,GTK_STATE_NORMAL,&black);
  gtk_widget_modify_base(WorldList[WorldListIndex].output_text,GTK_STATE_ACTIVE,&black);
  gtk_widget_modify_base(WorldList[WorldListIndex].output_text,GTK_STATE_PRELIGHT,&black);
  gtk_widget_modify_base(WorldList[WorldListIndex].output_text,GTK_STATE_SELECTED,&black);
  gtk_widget_modify_base(WorldList[WorldListIndex].output_text,GTK_STATE_INSENSITIVE,&black);
  gtk_widget_modify_text(WorldList[WorldListIndex].output_text,GTK_STATE_NORMAL,&white);
  gtk_widget_modify_font(WorldList[WorldListIndex].output_text,my_font);
  WorldList[WorldListIndex].curfont = my_font;
  gtk_container_add (GTK_CONTAINER (WorldList[WorldListIndex].output_scroller), WorldList[WorldListIndex].output_text);
  gtk_text_view_set_editable(GTK_TEXT_VIEW(WorldList[WorldListIndex].output_text),FALSE);
  gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(WorldList[WorldListIndex].output_text),GTK_WRAP_CHAR);
  gtk_widget_show (WorldList[WorldListIndex].output_text);

  WorldList[WorldListIndex].input_scroller = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (WorldList[WorldListIndex].input_scroller);
  gtk_paned_pack2 (GTK_PANED (WorldList[WorldListIndex].io_pane), WorldList[WorldListIndex].input_scroller, TRUE, TRUE);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (WorldList[WorldListIndex].input_scroller), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

  WorldList[WorldListIndex].input_text = gtk_text_view_new ();
  gtk_widget_show (WorldList[WorldListIndex].input_text);
  gtk_container_add (GTK_CONTAINER (WorldList[WorldListIndex].input_scroller), WorldList[WorldListIndex].input_text);

  WorldList[WorldListIndex].world_name = gtk_label_new (NULL);
  gtk_widget_show (WorldList[WorldListIndex].world_name);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (world_view), gtk_notebook_get_nth_page (GTK_NOTEBOOK (world_view), WorldListIndex), WorldList[WorldListIndex].world_name);
  gtk_label_set_justify (GTK_LABEL (WorldList[WorldListIndex].world_name), GTK_JUSTIFY_LEFT);


	gtk_notebook_set_page (GTK_NOTEBOOK (world_view), WorldListIndex);
	gtk_widget_grab_focus(GTK_WIDGET(WorldList[WorldListIndex].input_text));


	sprintf(WindowTitle, "Panache : %s", WorldList[WorldListIndex].DisplayName->str);
#ifdef DEBUG
	printf("Panache : %s\n", WorldList[WorldListIndex].DisplayName->str);
#endif


	gtk_label_set_text (GTK_LABEL (WorldList[WorldListIndex].world_name), WorldList[WorldListIndex].DisplayName->str);
	gtk_window_set_title (GTK_WINDOW (main_window), (gchar *)WindowTitle);

	g_signal_connect (GTK_TEXT_VIEW(WorldList[WorldListIndex].input_text),
		"key-press-event", GTK_SIGNAL_FUNC (scroll_history), NULL);

	g_signal_connect_after (gtk_text_view_get_buffer(GTK_TEXT_VIEW(WorldList[WorldListIndex].input_text)),
		"insert-text", GTK_SIGNAL_FUNC (catch_input), NULL);
/*		
	gtk_signal_connect (GTK_OBJECT (DisplayPanes),
		"switch-page", GTK_SIGNAL_FUNC (switch_worlds), NULL);
*/		
	WorldList[WorldListIndex].sockfd = socket(AF_INET, SOCK_STREAM, 0);

	printf("Socket : %d\n",WorldList[WorldListIndex].sockfd);
	
	WorldList[WorldListIndex].my_addr.sin_family = AF_INET;
	WorldList[WorldListIndex].my_addr.sin_port = htons(WorldList[WorldListIndex].WorldPort);
	
	sprintf(WorldDomain, "%s.", WorldList[WorldListIndex].WorldDomain->str);
	
	h = gethostbyname((const char *)WorldDomain);
		
	WorldList[WorldListIndex].my_addr.sin_addr = *((struct in_addr *)h->h_addr_list[0]);
	memset(&(WorldList[WorldListIndex].my_addr.sin_zero), '\0', 8);
	if (connect(WorldList[WorldListIndex].sockfd, (struct sockaddr *)&WorldList[WorldListIndex].my_addr, sizeof(struct sockaddr)) >= 0 ) {
		printf("Worked...\n");
		WorldList[WorldListIndex].io_channel = g_io_channel_unix_new(WorldList[WorldListIndex].sockfd);
		WorldList[WorldListIndex].network_read_handler = g_io_add_watch(WorldList[WorldListIndex].io_channel, G_IO_IN, network_read, GINT_TO_POINTER(WorldListIndex));
	} else {
		perror("connect");
		printf("Didn't Work...\n");	
	}
}
